/*
 * Created on: 11/03/05 15:42:31
 */
package edu.wpi.ejutr.ebob_observer;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.model.*;
import edu.wpi.ebob.model.cm.*;
import edu.wpi.ejutr.debug.DebugDialog;
import edu.wpi.ejutr.logger.EBOBActionLogger;
import edu.wpi.ejutr.wizards.*;

import java.util.*;

import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;

/**
 * EBOBListener implements the ObjectBenchListener interface, such that it can
 * attach to the object bench and receive notification whenever:
 * 		1. An object is added to the Bench.
 * 	    2. An object is removed from the Bench.
 *      3. A method within an object within the Bench is called.
 *      
 * Then it notifies an EBOBActionLogger which actually stores what objects exists
 * and which methods were called when.
 * 
 * Implemented as a singleton, so that we can get to the listener from anywhere
 * without it being static or tied to a particular parent object. 
 * 
 * @author Ben Mohlenhoff - <a href="mailto:bigben@wpi.edu">bigben@wpi.edu</a>
 */

public class EBOBListener implements ObjectBenchListener {	
	
	private EBOBActionLogger log; 
	private static EBOBListener me;
	private ObjectBench theBench;
	
	/**
	 * Private constructor, initializes the listener.
	 *
	 */
	private EBOBListener() {
		this.log = EBOBActionLogger.getInstance();
		me = this;
		theBench = ObjBenchPlugin.getDefault().getBench();
	}
	
	/**
	 * Returns a handle to the listener.
	 * @return a handle to the listener.
	 */
	public static EBOBListener getInstance() {
		if (me == null) {
			me = new EBOBListener();
		} 
		return me;
	}
	
	/**
	 * Triggers when an object is added to the bench and the listener is active.
	 */
	public void objectAdded(ObjectBenchEvent event) {		
		// debug
		DebugDialog.printToConsole("objectAdded: triggered\n");
		// event.getName() = instance name of the object
		String instanceName = event.getName();
		// event.getObject().toString() = fully qualified object name
		String fqon = event.getObject().objToString();
		
		fqon = fqon.substring(0, fqon.indexOf("@"));
		
		String message;
		message = "Debug: Object added to bench...\n" + 
				  "Object name: " + instanceName + "\n" + 
                  "Object type: " + fqon + "\n";
		
		DebugDialog.show(message);
		
		// now, to log it
		log.logObjectAddition(fqon, instanceName);
	}

	/**
	 * Triggers whenever an object is removed from the bench.
	 */
	public void objectRemoved(ObjectBenchEvent event) {
		// debug
		DebugDialog.printToConsole("objectRemoved: Triggered\n");
		// event.getName() = instance name of the object
		String instanceName = event.getName();
		// event.getObject().toString() = fully qualified object name
		String fqon = event.getObject().objToString();
		
		String message;
		message = "Debug: Object removed from bench...\n" + 
				  "Object name: " + instanceName + "\n" + 
		          "Object type: " + fqon + "\n";		
		DebugDialog.show(message);
		
		// log it
		log.logObjectRemoval(fqon, instanceName);
	}
	
	/**
	 * Triggers whenever a method is called.
	 */
	public void methodCalled(MethodInvokeInfo mii) {
		// debug
		DebugDialog.printToConsole("methodCalled: triggered\n");
		String methodName = mii.getFullyQualifiedMethodName();
		String retType = mii.getMethodReturnType();
		String callingObject = mii.getFullyQualifiedObjectName();			
		String instance = mii.getCallingObjectInstanceName();		
		String signature = "";
				
		Iterator it = mii.getParameterIterator();		
		
		DebugDialog.show("Debug: methodCalled was triggered. \n\n" +
				"Object Name: " + callingObject + "\n" + 				
				"Method Name: " + methodName + "\n" +
				"Return Type: " + retType + "\n" + 
				"Instance Name: " + instance + "\n");
		while (it.hasNext()) {
			MethodInvokeParamInfo mipi = (MethodInvokeParamInfo)it.next();
			
			if (signature.equals("")) {
				signature += mipi.getParamObjectInstanceName();
			} else {
				signature += ", " + mipi.getParamObjectInstanceName();
			}
			//System.out.println("objInstName=" + signature);
		}
		
		// log it
		log.logMethodCall(instance, methodName, signature, retType);
		
		// figure out assertions
		if (!retType.equals("void")) {
			//System.out.println("assertion wizard triggered\n");

			String methodCall = instance + "." + methodName + "()";
			// pop up dialogue for assertions
			Shell shell = new Shell();
			
			AssertionWizard assertWiz = new AssertionWizard(retType, methodCall);
			WizardDialog dialog = new WizardDialog(shell, assertWiz);
			
			if (dialog.open() == WizardDialog.OK) { // assertion needs to be logged				
				if (assertWiz.doAssert()) {
					log.logAssertion(assertWiz.getLHS(), assertWiz.getComparison(), assertWiz.getRHS(), assertWiz.getExpectedResult());
				}
			}			
		}
	}
	
	/**
	 * Attaches this listener to the object bench.
	 * @param sendCurrentStatus set this to true if you want to be notified of all objects currently in the bench.
	 */
	public void attach(boolean sendCurrentStatus) {
		//DebugDialog.show("Attaching listener...");
		theBench.addObjectBenchListener(this, sendCurrentStatus);
	}
	
	/**
	 * Detaches this listener from the object bench.
	 *
	 */
	public void detach() {
		//DebugDialog.show("Detaching listener...");
		theBench.removeObjectBenchListener(this);
	}

}
